import React, { useEffect, useRef } from 'react'
import Empty from './Empty'
import PropTypes from 'prop-types'

import ReactEchartsCore from 'echarts-for-react/lib/core'
import echarts from 'echarts/lib/echarts'
import 'echarts/lib/chart/pie'
import 'echarts/lib/component/legend'
import 'echarts/lib/component/legendScroll'
import 'echarts/lib/component/tooltip'
import 'echarts/lib/component/title'
import 'echarts/lib/component/markPoint'
import 'echarts/lib/component/markLine'
import 'echarts/lib/component/markArea'
import 'echarts/lib/component/dataZoom'

import { summate } from '../utils'

let index = 0 // 高亮索引

const PieChart = ({ chartData, radius, showTitle, ...rest }) => {
  const pieRef = useRef(null)
  useEffect(() => {
    const pieInstance = pieRef.current.getEchartsInstance()

    console.log('index', index)
    pieInstance.dispatchAction({
      type: 'highlight',
      seriesIndex: index,
      dataIndex: index,
    })
    pieInstance.on('mouseover', function(e) {
      if (e.dataIndex != index) {
        this.dispatchAction({
          type: 'downplay',
          seriesIndex: 0,
          dataIndex: index,
        })
      } else {
        this.dispatchAction({ type: 'downplay', seriesIndex: 0, dataIndex: 0 })
      }
    })

    pieInstance.on('mouseout', function(e) {
      index = e.dataIndex
      this.dispatchAction({
        type: 'highlight',
        seriesIndex: 0,
        dataIndex: e.dataIndex,
      })
    })
    return () => {
      pieInstance.off('mouseover')
      pieInstance.off('mouseout')
    }
  }, [chartData])

  const option = {
    title: {
      text: chartData.title.text,
      show: showTitle,
    },
    tooltip: {
      trigger: 'item',
      formatter: '{a} <br/>{b}: {c} ({d}%)',
    },
    legend: {
      top: 'bottom',
      type: 'scroll',
      orient: 'vertical',
      right: '10%',
      top: 'center',
      itemGap: 15,
      selectedMode: false,
      icon: 'pin',
      textStyle: {
        fontSize: 14,
        rich: {
          name: {
            color: '#262A30',
          },
          value: {
            padding: [0, 5, 0, 5],
          },
          percent: {
            color: '#00A9FF',
          },
        },
      },

      formatter: function(name) {
        const total = summate(chartData.series[0].data)
        const res = chartData.series[0].data.filter(v => v.name === name)
        const percent = ((res[0].value * 100) / total).toFixed(2)
        return '{name| ' + name + '} {value|' + res[0].value + '} {percent|' + percent + '%}'
      },
    },
    toolbox: {
      show: false,
    },

    series: [
      {
        name: chartData.series.length ? chartData.series[0].name : '',
        type: 'pie',
        radius,
        selectedMode: 'single',
        // center: ['50%', '60%'],
        data: chartData.series.length ? chartData.series[0].data : [],
        emphasis: {
          label: {
            show: true,
            fontSize: 20,
            fontWeight: 'bold',
            formatter: '{per|{d}%}\n{name|{b}}',
            textStyle: {
              rich: {
                per: {
                  fontSize: 18,
                  color: '#959BA3',
                  lineHeight: 30,
                },
                name: {
                  fontSize: 14,
                  fontWeight: 'normal',
                  color: '#41A6FE',
                },
              },
            },
          },
        },
        labelLine: {
          show: false,
        },
        label: {
          formatter: '{b} {d}% \n',
          show: false,
          padding: [0, 0],
          position: 'center',
        },
      },
    ],
  }

  if (!chartData.series.length) {
    return <Empty image={Empty.PRESENTED_IMAGE_SIMPLE} />
  }

  return <ReactEchartsCore echarts={echarts} notMerge option={option} {...rest} ref={pieRef} />
}

PieChart.propTypes = {
  theme: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  radius: PropTypes.array,
  showTitle: PropTypes.bool,
  style: PropTypes.object,
  className: PropTypes.string,
  chartData: PropTypes.object,
}

PieChart.defaultProps = {
  style: {},
  classname: '',
  theme: 'macarons',
  showTitle: false,
  radius: ['35%', '60%'],
  chartData: {
    title: { text: '' },
    series: [
      {
        center: ['25%', '30%'],
        data: [
          { value: 335, name: '直接访问' },
          { value: 310, name: '邮件营销' },
          { value: 234, name: '联盟广告' },
          { value: 135, name: '视频广告' },
          { value: 1548, name: '搜索引擎' },
        ],
      },
    ],
  },
}

export default PieChart
